#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Implement unit tests for charm-rsyslog."""

import os
import shutil
import tempfile
from os.path import abspath, dirname

import hooks

import six


__author__ = "Jorge Niedbalski R. <jorge.niedbalski@canonical.com>"
UNITTEST_DIR = dirname(abspath(__file__))
CHARM_DIR = dirname(dirname(UNITTEST_DIR))

try:
    import unittest  # noqa E402
    import mock  # noqa E402
except ImportError:
    raise ImportError("Please install unittest and mock modules")


TO_PATCH = [
    "apt_install",
    "install",
    "service_restart",
    "service_start",
    "open_port",
    "service_stop",
    "close_port",
    "juju_log",
    "charm_dir",
    "config_get",
    "nrpe",
    "lsb_release",
    "status_set",
]


class BaseTestCase(unittest.TestCase):
    """Base class for test case."""

    def setUp(self):
        """Initialize the test case."""
        unittest.TestCase.setUp(self)
        self.patch_all()

        self.juju_log.return_value = True
        self.apt_install.return_value = True
        self.charm_dir.return_value = CHARM_DIR
        self.tmpdir = tempfile.mkdtemp()
        self.maxDiff = None
        self.lsb_release.return_value = {"DISTRIB_CODENAME": "bionic"}

    def tearDown(self):
        """Remove test resources."""
        try:
            shutil.rmtree(self.tmpdir)
        except Exception:
            pass

    def patch(self, method):
        """Mock a method."""
        _m = mock.patch.object(hooks, method)
        _mock = _m.start()
        self.addCleanup(_m.stop)
        return _mock

    def patch_all(self):
        """Mock all methods in TO_PATCH."""
        for method in TO_PATCH:
            setattr(self, method, self.patch(method))


class TestHooks(BaseTestCase):
    """Test hooks lib."""

    def test_install_hook(self):
        """Check if install hooks is correctly executed."""
        hooks.hooks.execute(["install"])
        self.apt_install.assert_called_with(
            ["rsyslog", "rsyslog-relp", "python3-jinja2"], options=["-f"]
        )

    def test_upgrade_charm(self):
        """Check if charm upgrade hooks is correctly executed."""
        hooks.hooks.execute(["upgrade-charm"])
        self.install.assert_called()

    def test_start_charm(self):
        """Check if start hook is correctly executed."""
        hooks.hooks.execute(["start"])
        self.service_start.assert_called_with("rsyslog")

        expected = [mock.call(514, protocol="UDP")]

        self.assertEquals(self.open_port.call_args_list, expected)

    def test_stop_charm(self):
        """Check if start hooks is correctly executed."""
        hooks.hooks.execute(["stop"])

        self.service_stop.assert_called_with("rsyslog")

        expected = [mock.call(514, protocol="UDP")]

        self.assertTrue(self.close_port.call_args_list == expected)

    def _tpl_has_value(self, template, value):
        return (
            hooks.get_config_template(template)
            .render(**hooks.get_changed_config())
            .find(value)
            > 0  # noqa:W503
        )

    def test_config_nova_disabled(self):
        """Check if configuration templates are created correctly (nova_logs: False)."""
        self.apt_install.return_value = True
        self.charm_dir.return_value = CHARM_DIR

        config = {
            "forward_host": "",
            "forward_options": "",
            "forward_port": "514",
            "forward_protocol": "tcp",
            "forward_selector": "*.*",
            "syslog_rotate": 7,
            "messages_rotate": 4,
            "nova_logs": False,
            "protocol": "udp",
        }

        def config_side(*args, **kwargs):
            return config[args[0]]

        self.config_get.side_effect = config_side

        self.assertFalse(self._tpl_has_value("60-aggregator.conf", "nova"))
        self.assertTrue(self._tpl_has_value("rsyslog.conf", "7"))
        self.assertTrue(self._tpl_has_value("rsyslog.conf", "4"))

    def test_config_nova_enabled(self):
        """Check if configuration templates are created correctly (nova_logs: True)."""
        config = {
            "forward_host": "",
            "forward_options": "",
            "forward_port": "514",
            "forward_protocol": "tcp",
            "forward_selector": "*.*",
            "syslog_rotate": 7,
            "messages_rotate": 4,
            "nova_logs": True,
            "protocol": "udp",
        }

        def config_side(*args, **kwargs):
            return config[args[0]]

        self.config_get.side_effect = config_side
        self.assertTrue(self._tpl_has_value("60-aggregator.conf", "nova"))

    def test_config_changed(self):
        """Check if config-changed is executed correctly."""
        _open = mock.mock_open(read_data=b"foo")
        lsb = hooks.lsb_release()

        if six.PY2:
            open_function = "__builtin__.open"
        else:
            open_function = "builtins.open"

        with mock.patch(open_function, _open, create=True), mock.patch.object(
            hooks, "validate_string", return_value=True
        ), mock.patch.object(hooks, "lsb_release") as mock_lsb:
            mock_lsb.return_value = lsb
            hooks.config_changed()

        # I'm not quite sure why config_changed appears to be called twice but
        # it does behave properly and only make changes once
        expected = [
            mock.call(
                os.path.join(hooks.get_template_dir(), "60-aggregator.conf"), "rb"
            ),
            mock.call(
                os.path.join(hooks.get_template_dir(), "60-aggregator.conf"), "rb"
            ),
            mock.call(
                os.path.join(hooks.DEFAULT_RSYSLOG_PATH, "60-aggregator.conf"), "w"
            ),
            mock.call(os.path.join(hooks.get_template_dir(), "70-forward.conf"), "rb"),
            mock.call(os.path.join(hooks.get_template_dir(), "70-forward.conf"), "rb"),
            mock.call(os.path.join(hooks.DEFAULT_RSYSLOG_PATH, "70-forward.conf"), "w"),
            mock.call(os.path.join(hooks.get_template_dir(), "rsyslog.conf"), "rb"),
            mock.call(os.path.join(hooks.get_template_dir(), "rsyslog.conf"), "rb"),
            mock.call(hooks.DEFAULT_LOGROTATE_PATH, "w"),
        ]

        self.assertEquals(sorted(_open.call_args_list), sorted(expected))
        self.service_restart.assert_called_once_with("rsyslog")

    def test_string_validate(self):
        """Check that we can do protocol validation."""
        self.assertTrue(hooks.validate_string("a", ("a", "b", "c")))
        self.assertFalse(hooks.validate_string("d", ("a", "b", "c")))
        self.assertTrue(hooks.validate_string(None, ("a", "b", "c")))


class TestSeries(BaseTestCase):
    """Test different ubuntu series."""

    def setUp(self):
        """Initialize test environment."""
        super(TestSeries, self).setUp()
        self._logrotate_path = hooks.DEFAULT_LOGROTATE_PATH
        self._rsyslog_path = hooks.DEFAULT_RSYSLOG_PATH

    def tearDown(self):
        """Clean up the test."""
        hooks.DEFAULT_LOGROTATE_PATH = self._logrotate_path
        hooks.DEFAULT_RSYSLOG_PATH = self._rsyslog_path
        super(TestSeries, self).tearDown()

    def test_series(self):
        """Test different ubuntu series."""
        rsyslog_config = os.path.join(self.tmpdir, "rsyslog.conf")
        hooks.DEFAULT_LOGROTATE_PATH = rsyslog_config
        hooks.DEFAULT_RSYSLOG_PATH = os.path.join(self.tmpdir)
        lsb = hooks.lsb_release()
        with mock.patch.object(
            hooks, "validate_string", return_value=True
        ), mock.patch.object(hooks, "lsb_release") as mock_lsb:
            # bionic
            lsb["DISTRIB_CODENAME"] = "bionic"
            mock_lsb.return_value = lsb
            hooks.config_changed()

            with open(rsyslog_config, "r") as f:
                content = f.read()
                self.assertIn("invoke-rc.d rsyslog rotate > /dev/null", content)

            # xenial
            lsb["DISTRIB_CODENAME"] = "xenial"
            mock_lsb.return_value = lsb
            hooks.config_changed()

            with open(rsyslog_config, "r") as f:
                content = f.read()
                self.assertIn("invoke-rc.d rsyslog rotate > /dev/null", content)

            # trusty
            lsb["DISTRIB_CODENAME"] = "trusty"
            hooks.config_changed()
            with open(rsyslog_config, "r") as f:
                content = f.read()
                self.assertIn("reload rsyslog >/dev/null 2>&1 || true", content)


if __name__ == "__main__":
    unittest.main()
